import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.*;
import javax.swing.*;

import sv.ui.*;
import sv.util.*;


/** 
 * ContourDialog.java
 * Manage and configure the Contour
 * @version 1.0
 * @author Byeongseob Ki
 */

public class ContourDialog extends Dialog 
                          implements ActionListener, ItemListener {

  // user input fileld
  private JSlider lineslider;
  private TextField minfield;
  private TextField maxfield;

  private int lines;
  private float min, max;

  private Checkbox labelCheck; 
  private boolean isLabel;

  // to access GFrame2D class as parent
  protected GFrame2D gframe;

  public ContourDialog(Frame gframe, int lines, float min, float max) {
     super(gframe, "Contour Configuration", false);
     this.gframe = (GFrame2D)gframe;
  
     this.lines = lines;
     this.min = min;
     this.max = max;

     // make GUI components
     makeGUI();
  }


  /**
   * overrided method (for buttons)
   */
  public void actionPerformed(ActionEvent evt) {
     String command = evt.getActionCommand();
     Float tmin, tmax;
     if (command.equals("Apply")) {    
        int lines = lineslider.getValue();
        gframe.setContourLines(lineslider.getValue());        
        tmin = new Float(minfield.getText());
        tmax = new Float(maxfield.getText());
        gframe.setContourMin(tmin.floatValue());   
        gframe.setContourMax(tmax.floatValue());  

        float zmax = tmax.floatValue();
        float zmin = tmin.floatValue();
        float[] values =  new float[lines];
        for (int i=0; i < lines; i++) 
          values[i] = (float)((float)(i)/lines*(zmax-zmin)+zmin);
 
        gframe.canvas.setContourValues(values, true);

     } else if (command.equals("User Mode")) {    
        UserContourDialog contourDialog = 
           new UserContourDialog(gframe, lines, min, max);
        contourDialog.show();   
        this.dispose();
     } else if (command.equals("Quit ")) {       
       this.dispose();
     }
  }


  /**
   * overrided function
   */
  public void itemStateChanged(ItemEvent evt) {
     String itemName  = evt.getItem().toString();

     if (itemName.equals("Draw Contour Label")) 
        gframe.canvas.drawContourLabel(labelCheck.getState());
  }

  // make GUI of this class
  private void makeGUI() {

     setLayout(new BorderLayout());
     setSize(200, 360);

     Panel centerPanel = new Panel();
     Panel lowerPanel = new Panel();
     Panel dummy;
     JPanel p;

     centerPanel.setLayout(new GridLayout(3, 1));

     p = new JPanel();
     p.setLayout(new GridLayout(2, 1));
     p.setBorder(new TitledBorder("Contour Lines & Label: "));

     Panel slideP = new Panel();
     slideP.setLayout(new BorderLayout());
     slideP.add("Center", 
               lineslider = new JSlider(JSlider.HORIZONTAL, 0, 30, lines));
     slideP.add("East", slideValue = new Label(String.valueOf(lines)));
     p.add(slideP);
     createSlideListener();

     p.add(labelCheck = new Checkbox("Draw Contour Label"));
     labelCheck.addItemListener(this);
     labelCheck.setState(false); 

     centerPanel.add(p);

     p = new JPanel();
     p.setLayout(new BorderLayout());
     p.setBorder(new TitledBorder("Minimum Values: "));	

     dummy = new Panel(); 
     dummy.add(new Label("Set minimum values:", Label.LEFT));
     p.add("North", dummy);
     
     p.add("Center", minfield = new TextField(String.valueOf(min)));
     centerPanel.add(p);

     p = new JPanel();
     p.setLayout(new BorderLayout());
     p.setBorder(new TitledBorder("Maximum Values: "));	

     dummy = new Panel(); 
     dummy.add(new Label("Set maximum values:", Label.LEFT));
     p.add("North", dummy);
     p.add("Center", maxfield = new TextField(String.valueOf(max)));
     centerPanel.add(p);

     // make lower part(two buttons)
     lowerPanel.setLayout(new BorderLayout());
     lowerPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));
     Button applybtn = new Button("Apply");
     applybtn.addActionListener(this);
     lowerPanel.add(applybtn);

     Button userbtn = new Button("User Mode");
     userbtn.addActionListener(this);
     lowerPanel.add(userbtn);

     Button quitbtn = new Button("Quit ");
     quitbtn.addActionListener(this);
     lowerPanel.add(quitbtn);

     // add all parts
     add("Center", centerPanel);
     add("South", lowerPanel);
  }

  private Label slideValue;

  // slide value changes
  void createSlideListener() {
     ChangeListener changeListener = new ChangeListener() {
        public void stateChanged(ChangeEvent e) {
           slideValue.setText(String.valueOf(lineslider.getValue()));
        }
     };
     lineslider.addChangeListener(changeListener);
  }
 
 
}



class UserContourDialog extends JDialog 
                          implements ActionListener {

  // user input fileld
  private TextField[] valuefield;

  private int lines;
  private float min, max;

  private Panel p1, p2, p3;

  // to access GFrame2D class as parent
  protected GFrame2D gframe;

  public UserContourDialog(Frame gframe, int lines, float min, float max) {
     super(gframe, "Contour Configuration", false);
     this.gframe = (GFrame2D)gframe;
  
     valuefield = new TextField[30];
     this.lines = lines;
     this.min = min;
     this.max = max;

     // make GUI components
     makeGUI();
  }


  /**
   * overrided method (for buttons)
   */
  public void actionPerformed(ActionEvent evt) {
     String command = evt.getActionCommand();

     String value;
     int index  = 0;
     if (command.equals("Apply")) {     

       Vector valVec = new Vector();
       for (int i=0; i<30; i++) {
          value = valuefield[i].getText().trim();
          if (value.length() > 0) {
             try {
               valVec.addElement(new Float(value));
             } catch (Exception e) {
                System.out.println("Number format error: " + value);
                return;
             } 
          }
       }
       float[] val = new float[valVec.size()];
       for (int i=0; i<valVec.size(); i++) 
          val[i] = ((Float)valVec.elementAt(i)).floatValue();
       gframe.canvas.setContourValues(val, true);

     } else if (command.equals("Log Step")) {  
       for (int i=0; i<30; i++) {
          value = valuefield[i].getText().trim();
          if (value.length() > 0) 
             index++;
       }
       lines = index;

       setFields(2);

     } else if (command.equals("Random")) {    
       for (int i=0; i<30; i++) {
          value = valuefield[i].getText().trim();
          if (value.length() > 0) 
             index++;
       }
       lines = index;

       setFields(3);

     } else if (command.equals("Quit")) {       
       this.dispose();
     }
  }

  // set values in each text fields
  private void setFields(int type) {
    switch (type) {
       case 1:
	 for (int i=0; i<lines; i++) {
            float label = (float)((float)(i)/lines*(max-min)+min);
            valuefield[i].setText(Float.toString(label));
	 }
         break;
       case 2:
	 for (int i=0; i<lines; i++) {
            float inc = (float)(Math.log(max-min)/(double)(lines-1));
            float label = (float)(min + Math.pow(Math.E,(double)(i+1)*inc)); 
            valuefield[i].setText(Float.toString(label));
	 }
         break;
       case 3:
	 for (int i=0; i<lines; i++) {
            float label = (float)(Math.random()*(max-min));
            valuefield[i].setText(Float.toString(label));
	 }
         break;
    }
    for (int i=lines; i<30; i++) 
            valuefield[i].setText(""); 
  }

  // make GUI of this class
  private void makeGUI() {

     setLayout(new BorderLayout());
     setSize(260, 420);

     Panel lowerPanel = new Panel();
     Panel dummy;
     Button btn;

     JTabbedPane centerPanel = new JTabbedPane();

     centerPanel.addTab("1-10",  p1 = new Panel());
     centerPanel.setSelectedIndex(0);
     centerPanel.addTab("11-20",  p2 = new Panel());
     centerPanel.addTab("21-30",  p3 = new Panel());

     p1.setLayout(new GridLayout(10, 1));
     p2.setLayout(new GridLayout(10, 1));
     p3.setLayout(new GridLayout(10, 1));
     for (int i=0; i<10; i++) {
        dummy = new Panel();
        dummy.setLayout(new BorderLayout());
        dummy.add("West", new Label(Integer.toString(i)));
        dummy.add("Center", valuefield[i] = new TextField());
        p1.add(dummy);

        dummy = new Panel();
        dummy.setLayout(new BorderLayout());
        dummy.add("West", new Label(Integer.toString(i+10)));
        dummy.add("Center", valuefield[i+10] = new TextField());
        p2.add(dummy);

        dummy = new Panel();
        dummy.setLayout(new BorderLayout());
        dummy.add("West", new Label(Integer.toString(i+20)));
        dummy.add("Center", valuefield[i+20] = new TextField());
        p3.add(dummy);
     }
     setFields(1);

     btn = new Button("Log Step");
     btn.addActionListener(this);
     lowerPanel.add(btn);

     btn = new Button("Random");
     btn.addActionListener(this);
     lowerPanel.add(btn);

     btn = new Button("Apply");
     btn.addActionListener(this);
     lowerPanel.add(btn);

     btn = new Button("Quit");
     btn.addActionListener(this);
     lowerPanel.add(btn);

     // add all parts
     add("Center", centerPanel);
     add("South", lowerPanel);
  }
 
}

